@@ -25,7 +25,7 @@
 #include <string.h>
 #include <strings.h>
 #include <popt.h>
-#include <gdk/gdkwindow.h>
+#include <gdk/gdk.h>
 #include <libbonobo.h>
 #include <login-helper/login-helper.h>
 #include <gdk/gdkx.h>
@@ -40,6 +40,14 @@
 #include "zoom-region-private.h"
 #include "GNOME_Magnifier.h"
 
+#undef DEBUG_CLIENT_CALLS
+
+#ifdef DEBUG_CLIENT_CALLS
+#define DBG(a) a
+#else
+#define DBG(a)
+#endif
+
 typedef struct
 {
     LoginHelper parent;
@@ -84,6 +92,24 @@ enum {
 	MAGNIFIER_CROSSWIRE_COLOR_PROP
 } PropIdx;
 
+#ifdef DEBUG_CLIENT_CALLS
+gchar* mag_prop_names[MAGNIFIER_CROSSWIRE_COLOR_PROP + 1] = {
+    "SOURCE_DISPLAY",
+    "TARGET_DISPLAY",
+    "SOURCE_SIZE",
+    "TARGET_SIZE",
+    "CURSOR_SET",
+    "CURSOR_SIZE",
+    "CURSOR_ZOOM",
+    "CURSOR_COLOR",
+    "CURSOR_HOTSPOT",
+    "CURSOR_DEFAULT_SIZE",
+    "CROSSWIRE_SIZE",
+    "CROSSWIRE_CLIP",
+    "CROSSWIRE_COLOR"
+};
+#endif
+
 static int _x_error = 0;
 static int fixes_event_base = 0, fixes_error_base;
 static Display *cursor_client_connection;
@@ -93,6 +119,7 @@ static void magnifier_transform_cursor (Magnifier *magnifier);
 static void magnifier_init_cursor_set (Magnifier *magnifier, gchar *cursor_set);
 static gboolean magnifier_check_set_struts (Magnifier *magnifier);
 static gboolean magnifier_reset_struts_at_idle (gpointer data);
+static void magnifier_init_window (Magnifier *magnifier, GdkScreen *screen);
 static gboolean _is_override_redirect = FALSE;
 
 static Window*
@@ -257,9 +284,12 @@ magnifier_set_cursor_from_pixbuf (Magnifier *magnifier, GdkPixbuf *cursor_pixbuf
 		height = gdk_pixbuf_get_height (cursor_pixbuf);
 		pixmap = gdk_pixmap_new (drawable, width, height, -1);
 		gc = gdk_gc_new (pixmap);
-		gdk_draw_pixbuf (pixmap, gc, cursor_pixbuf, 0, 0, 0, 0, 
-				 width, height,
-				 GDK_RGB_DITHER_NONE, 0, 0);
+		if (GDK_IS_DRAWABLE (pixmap))
+		    gdk_draw_pixbuf (pixmap, gc, cursor_pixbuf, 0, 0, 0, 0, 
+				     width, height,
+				     GDK_RGB_DITHER_NONE, 0, 0);
+		else
+		    DBG (g_warning ("empty cursor pixmap created."));
 		mask = gdk_pixmap_new (drawable, width, height, 1);
 		gdk_pixbuf_render_threshold_alpha (cursor_pixbuf, mask, 0, 0, 0, 0, 
 						   width, height,
@@ -320,7 +350,6 @@ GdkPixbuf *
 magnifier_get_pixbuf_for_name (Magnifier *magnifier, const gchar *cursor_name)
 {
     GdkPixbuf *retval = NULL;
-    g_message ("cursor %s", cursor_name);
     if (magnifier->priv->cursorlist) 
 	    retval = g_hash_table_lookup (magnifier->priv->cursorlist, cursor_name);
     if (retval) 
@@ -443,6 +472,7 @@ magnifier_notify_damage (Magnifier *magnifier, XRectangle *rect)
 	rect_bounds.y2 = rect->y + rect->height;
 #ifdef DEBUG_DAMAGE
 	g_message ("damage");
+	g_message ("dirty %d, %d to %d, %d", rect_bounds.x1, rect_bounds.y1, rect_bounds.x2, rect_bounds.y2);
 #endif
 	magnifier_zoom_regions_mark_dirty (magnifier, rect_bounds);
 }
@@ -475,6 +505,7 @@ magnifier_set_extension_listeners (Magnifier *magnifier, GdkWindow *root)
 	    g_warning ("Damage client hooks were not initialized.");
 	if (!magnifier_cursor_notification_init (magnifier))
 	    g_warning ("Cursor change notification hooks were not initialized.");
+	magnifier->source_initialized = TRUE;
 }
 
 static void
@@ -807,17 +838,24 @@ magnifier_transform_cursor (Magnifier *magnifier)
 			magnifier->priv->w->window,
 			size_x, size_y,
 			-1);
-		if (!magnifier->priv->cursor) g_warning ("magnifier cursur pixmap is non-null");
+		if (!GDK_IS_DRAWABLE (magnifier->priv->cursor)) 
+		{
+		    DBG (g_warning ("NULL magnifier cursor pixmap."));
+		    return;
+		}
 		magnifier->priv->cursor_mask = gdk_pixmap_new (
 			magnifier->priv->w->window,
 			size_x, size_y,
 			1);
-
-		gdk_draw_pixbuf (magnifier->priv->cursor,
-				 cgc,
-				 scaled_cursor_pixbuf,
-				 0, 0, 0, 0, size_x, size_y,
-				 GDK_RGB_DITHER_NONE, 0, 0 );
+		if (GDK_IS_DRAWABLE (magnifier->priv->cursor)) {
+		    gdk_draw_pixbuf (magnifier->priv->cursor,
+				     cgc,
+				     scaled_cursor_pixbuf,
+				     0, 0, 0, 0, size_x, size_y,
+				     GDK_RGB_DITHER_NONE, 0, 0 );
+		}
+		else
+		    DBG (g_warning ("cursor pixmap is non-drawable."));
 		scaled_mask_pixbuf = gdk_pixbuf_add_alpha (
 			scaled_mask_pixbuf, True, 0, 0, 0);
 		gdk_pixbuf_render_threshold_alpha (scaled_mask_pixbuf,
@@ -869,7 +907,6 @@ magnifier_init_cursor_set (Magnifier *magnifier, gchar *cursor_set)
 		cursor_dirname = g_strconcat (CURSORSDIR, "/", magnifier->cursor_set, NULL);
 		cursor_dir = g_dir_open (cursor_dirname, 0, NULL);
 		/* assignment, not comparison, is intentional */
-		g_message ("cursor_dir=%x", cursor_dir);
 		while (cursor_dir && (filename = g_dir_read_name (cursor_dir)) != NULL) 
 		{
 			if (filename) 
@@ -1010,6 +1047,8 @@ magnifier_get_property (BonoboPropertyBag *bag,
 	Magnifier *magnifier = user_data;
 	GNOME_Magnifier_RectBounds rect_bounds;
 	int csize = 0;
+
+	DBG (fprintf (stderr, "Get property: \t%s\n", mag_prop_names[arg_id]));
 	
 	switch (arg_id) {
 	case MAGNIFIER_SOURCE_SIZE_PROP:
@@ -1080,7 +1119,7 @@ magnifier_set_property (BonoboPropertyBag *bag,
 	Magnifier *magnifier = user_data;
 	GNOME_Magnifier_RectBounds rect_bounds;
 	gchar *full_display_string;
-	
+
 	switch (arg_id) {
 	case MAGNIFIER_SOURCE_DISPLAY_PROP:
 		full_display_string = BONOBO_ARG_GET_STRING (arg);
@@ -1102,6 +1141,7 @@ magnifier_set_property (BonoboPropertyBag *bag,
 		    magnifier_get_display_rect_bounds (magnifier, &magnifier->source_bounds, FALSE);
 		    magnifier_check_set_struts (magnifier);
 		}
+		DBG (fprintf (stderr, "Set source display: \t%s\n", full_display_string));
 		break;
 	case MAGNIFIER_TARGET_DISPLAY_PROP:
 		full_display_string = BONOBO_ARG_GET_STRING (arg);
@@ -1115,22 +1155,43 @@ magnifier_set_property (BonoboPropertyBag *bag,
 			gdk_display_open (full_display_string);
 		    magnifier->target_display_name = g_strdup (full_display_string);
 		    if (GTK_IS_WINDOW (magnifier->priv->w)) 
+		    {
+#ifdef REPARENT_GTK_WINDOW_WORKS
 			gtk_window_set_screen (GTK_WINDOW (magnifier->priv->w), 
 					       gdk_display_get_screen (
 						   magnifier->target_display,
 						   magnifier->target_screen_num));
+#else
+			/* disconnect from the old window's destroy signal */
+			g_object_disconnect (magnifier->priv->w,
+				  "any_signal::realize", magnifier_realize, NULL,
+				  "any_signal::size_allocate", magnifier_size_allocate, NULL,
+				  "any_signal::destroy", magnifier_exit, NULL,
+			          NULL);
+			/* destroy the old windows */
+			gtk_widget_destroy (magnifier->priv->w);
+			/* and re-initialize... */
+			magnifier_init_window (magnifier, gdk_display_get_screen (
+						   magnifier->target_display,
+						   magnifier->target_screen_num));
+#endif
+		    }
 		    magnifier_get_display_rect_bounds (magnifier, &magnifier->target_bounds, TRUE);
 		    magnifier_get_display_rect_bounds (magnifier, &magnifier->source_bounds, FALSE);
 		    magnifier_reparent_zoom_regions (magnifier);
 		    magnifier_init_cursor_set (magnifier, magnifier->cursor_set); /* needed to reset pixmaps */
 		    magnifier_check_set_struts (magnifier);
 		}
+		DBG (fprintf (stderr, "Set target display: \t%s (screen %d)\n", 
+			      full_display_string, magnifier->target_screen_num));
 		break;
 	case MAGNIFIER_SOURCE_SIZE_PROP:
 	        magnifier->source_bounds = BONOBO_ARG_GET_GENERAL (arg,
 								   TC_GNOME_Magnifier_RectBounds,
 								   GNOME_Magnifier_RectBounds,
 								   NULL);
+		DBG (fprintf (stderr, "Set source size: \t%d,%d to %d,%d\n", 
+			      magnifier->source_bounds.x1, magnifier->source_bounds.y1, magnifier->source_bounds.x2, magnifier->source_bounds.y2));
 		break;
 	case MAGNIFIER_TARGET_SIZE_PROP:
 	        magnifier->target_bounds = BONOBO_ARG_GET_GENERAL (arg,
@@ -1145,17 +1206,22 @@ magnifier_set_property (BonoboPropertyBag *bag,
 				   magnifier->target_bounds.x2 - magnifier->target_bounds.x1,
 				   magnifier->target_bounds.y2 - magnifier->target_bounds.y1);
 		magnifier_check_set_struts (magnifier);
+		DBG (fprintf (stderr, "Set target size: \t%d,%d to %d,%d\n", 
+			      magnifier->target_bounds.x1, magnifier->target_bounds.y1, magnifier->target_bounds.x2, magnifier->target_bounds.y2));
 		break;
 	case MAGNIFIER_CURSOR_SET_PROP:
 		magnifier_init_cursor_set (magnifier, g_strdup (BONOBO_ARG_GET_STRING (arg)));
+		DBG (fprintf (stderr, "Setting cursor set: \t%s\n", BONOBO_ARG_GET_STRING (arg)));
 		break;
 	case MAGNIFIER_CURSOR_SIZE_PROP:
 		magnifier->cursor_size_x = BONOBO_ARG_GET_INT (arg);
 		magnifier->cursor_size_y = BONOBO_ARG_GET_INT (arg);
 		magnifier_transform_cursor (magnifier);
+		DBG (fprintf (stderr, "Setting cursor size: \t%d\n", magnifier->cursor_size_x));
 		break;
 	case MAGNIFIER_CURSOR_ZOOM_PROP:
 		magnifier->cursor_scale_factor = BONOBO_ARG_GET_FLOAT (arg);
+		DBG (fprintf (stderr, "Setting cursor scale factor: \t%f\n", (float) magnifier->cursor_scale_factor));
 		magnifier_transform_cursor (magnifier);
 		break;
 	case MAGNIFIER_CURSOR_COLOR_PROP:
@@ -1164,6 +1230,7 @@ magnifier_set_property (BonoboPropertyBag *bag,
 								  CORBA_unsigned_long, 
 								  NULL);
 		magnifier_transform_cursor (magnifier);
+		DBG (fprintf (stderr, "Setting cursor color: \t%u\n", (unsigned) magnifier->cursor_color));
 		break;
 	case MAGNIFIER_CURSOR_HOTSPOT_PROP:
 		magnifier->cursor_hotspot = BONOBO_ARG_GET_GENERAL (arg,
@@ -1179,13 +1246,16 @@ magnifier_set_property (BonoboPropertyBag *bag,
 		break;
 	case MAGNIFIER_CROSSWIRE_SIZE_PROP:
 		magnifier->crosswire_size = BONOBO_ARG_GET_INT (arg);
+		DBG (fprintf (stderr, "Setting crosswire size: \t%d\n", magnifier->crosswire_size));
 		/* TODO: notify zoomers */
 		break;
 	case MAGNIFIER_CROSSWIRE_CLIP_PROP:
 		magnifier->crosswire_clip = BONOBO_ARG_GET_BOOLEAN (arg);
+		DBG (fprintf (stderr, "Setting crosswire clip: \t%s\n", magnifier->crosswire_clip ? "true" : "false"));
 		break;
 	case MAGNIFIER_CROSSWIRE_COLOR_PROP:
 		magnifier->crosswire_color = BONOBO_ARG_GET_LONG (arg);
+		DBG (fprintf (stderr, "Setting crosswire size: \t%ld\n", (long) magnifier->crosswire_color));
 		break;
 	default:
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
@@ -1223,15 +1293,23 @@ impl_magnifier_set_source_display (PortableServer_Servant servant,
 	BonoboArg *arg = bonobo_arg_new (BONOBO_ARG_STRING);
 	BONOBO_ARG_SET_STRING (arg, display);
 	
-	magnifier_set_property (magnifier->property_bag,
-				arg,
-				MAGNIFIER_SOURCE_DISPLAY_PROP,
-				ev,
-				magnifier);
-	
-	/* attach listeners for DAMAGE, "dirty region", XFIXES cursor changes */
-	magnifier_set_extension_listeners (magnifier, magnifier_get_root (magnifier));
+	DBG (fprintf (stderr, "Set source display: \t%s\n", display));
 
+	if (strcmp (display, magnifier->source_display_name)) {
+
+	    magnifier_set_property (magnifier->property_bag,
+				    arg,
+				    MAGNIFIER_SOURCE_DISPLAY_PROP,
+				    ev,
+				    magnifier);
+	    /* attach listeners for DAMAGE, "dirty region", XFIXES cursor changes */
+	    magnifier_set_extension_listeners (magnifier, magnifier_get_root (magnifier));
+	}
+	else
+	{
+	    DBG (fprintf (stderr, "Attempt to set source to same value as previous: %s\n",
+			  display));
+	}
 	bonobo_arg_release (arg);
 }
 
@@ -1243,13 +1321,22 @@ impl_magnifier_set_target_display (PortableServer_Servant servant,
 	Magnifier *magnifier = MAGNIFIER (bonobo_object_from_servant (servant));
 	BonoboArg *arg = bonobo_arg_new (BONOBO_ARG_STRING);
 	BONOBO_ARG_SET_STRING (arg, display);
+	
+	DBG (fprintf (stderr, "Set target display: \t%s\n", display));
 
-	magnifier_set_property (magnifier->property_bag,
-				arg,
-				MAGNIFIER_TARGET_DISPLAY_PROP,
-				ev,
-				magnifier);
-
+	if (strcmp (display, magnifier->source_display_name)) 
+	{
+	    magnifier_set_property (magnifier->property_bag,
+				    arg,
+				    MAGNIFIER_TARGET_DISPLAY_PROP,
+				    ev,
+				    magnifier);
+	}
+	else
+	{
+	    DBG (fprintf (stderr, "Attempt to set source to same value as previous: %s\n",
+			  display));
+	}
 	bonobo_arg_release (arg);
 }
 
@@ -1259,6 +1346,8 @@ impl_magnifier_get_source_display (PortableServer_Servant servant,
 				   CORBA_Environment *ev)
 {
         Magnifier *magnifier = MAGNIFIER (bonobo_object_from_servant (servant));
+	DBG (fprintf (stderr, "Get source display: \t%s\n", magnifier->source_display_name));
+
 	return CORBA_string_dup (magnifier->source_display_name ? magnifier->source_display_name : "");
 }
 
@@ -1268,6 +1357,9 @@ impl_magnifier_get_target_display (PortableServer_Servant servant,
 				   CORBA_Environment *ev)
 {
         Magnifier *magnifier = MAGNIFIER (bonobo_object_from_servant (servant));
+	DBG (fprintf (stderr, "Get target display: \t%s\n", 
+		      magnifier->source_display_name));
+
 	return CORBA_string_dup (magnifier->source_display_name ? magnifier->target_display_name : "");
 }
 
@@ -1285,6 +1377,8 @@ impl_magnifier_create_zoom_region (PortableServer_Servant servant,
 	Bonobo_PropertyBag properties;
 	GNOME_Magnifier_ZoomRegion retval;
 
+	DBG (fprintf (stderr, "Create zoom region: \tzoom %f,%f, viewport %d,%d to %d,%d\n", (float) zx, (float) zy, viewport->x1, viewport->y1, viewport->x2, viewport->y2));
+
 	/* FIXME:
 	 * shouldn't do this here, since it causes the region to get
 	 * mapped onto the parent, if if it's not explicitly added!
@@ -1334,7 +1428,7 @@ impl_magnifier_add_zoom_region (PortableServer_Servant servant,
 {
 	Magnifier *magnifier = MAGNIFIER (bonobo_object_from_servant (servant));
 
-	if (magnifier->zoom_regions == NULL) 
+	if (!magnifier->source_initialized) 
 	{
 		magnifier_set_extension_listeners (magnifier, magnifier_get_root (magnifier));
 	}
@@ -1351,7 +1445,6 @@ impl_magnifier_get_properties (PortableServer_Servant servant,
 			       CORBA_Environment *ev)
 {
 	Magnifier *magnifier = MAGNIFIER (bonobo_object_from_servant (servant));
-
 	return bonobo_object_dup_ref (
 		BONOBO_OBJREF (magnifier->property_bag), ev);
 }
@@ -1378,6 +1471,8 @@ impl_magnifier_get_zoom_regions (PortableServer_Servant servant,
 			CORBA_Object_duplicate (objref, ev);
 	}
 	CORBA_sequence_set_release (list, CORBA_TRUE);
+
+	DBG (fprintf (stderr, "Get zoom regions: \t%d\n", len));
 	
 	return list; 
 }
@@ -1387,8 +1482,8 @@ impl_magnifier_clear_all_zoom_regions (PortableServer_Servant servant,
 				       CORBA_Environment * ev)
 {
 	Magnifier *magnifier = MAGNIFIER (bonobo_object_from_servant (servant));
+	DBG (fprintf (stderr, "Clear all zoom regions.\n"));
 
-	fprintf (stderr, "Destroying all zoom regions!\n");
 	g_list_foreach (magnifier->zoom_regions,
 			magnifier_unref_zoom_region, magnifier);
 	g_list_free (magnifier->zoom_regions);
@@ -1580,7 +1675,7 @@ magnifier_properties_init (Magnifier *magnifier)
 }
 
 static void
-magnifier_init_window (Magnifier *magnifier)
+magnifier_init_window (Magnifier *magnifier, GdkScreen *screen)
 {
 	GtkWindowType mag_win_type = GTK_WINDOW_TOPLEVEL;
 	if (_is_override_redirect) mag_win_type = GTK_WINDOW_POPUP;
@@ -1599,6 +1694,7 @@ magnifier_init_window (Magnifier *magnifier)
 				  "signal::size_allocate", magnifier_size_allocate, NULL,
 				  "signal::destroy", magnifier_exit, NULL,
 				  NULL);
+	gtk_window_set_screen (GTK_WINDOW (magnifier->priv->w), screen);
 	magnifier->priv->canvas = gtk_fixed_new ();
 	gtk_container_add (GTK_CONTAINER (magnifier->priv->w),
 			   magnifier->priv->canvas);
@@ -1628,7 +1724,9 @@ magnifier_init (Magnifier *magnifier)
 	magnifier->priv->w = NULL;
 	magnifier->priv->use_source_cursor = TRUE;
 	magnifier->priv->cursorlist = NULL;
-	magnifier_init_window (magnifier);
+	magnifier_init_window (magnifier, 
+			       gdk_display_get_screen (magnifier->target_display, 
+						       magnifier->target_screen_num));
 	magnifier_init_cursor_set (magnifier, "default");
 }
 
@@ -1678,8 +1776,6 @@ magnifier_new (gboolean override_redirect)
 		MAGNIFIER_OAFIID, BONOBO_OBJREF (mag));
 	if (ret != Bonobo_ACTIVATION_REG_SUCCESS)
 	    g_error ("Error registering magnifier server.\n");
-	else
-	    g_message ("The magnifier server was successfully registred.\n");
 
 	g_idle_add (magnifier_reset_struts_at_idle, mag);
 
